from enum import Enum


class MyEnum(Enum):
    A = 0
    B = 1
    C = 2


class ExampleClass():
    def a(self, a: int) -> int:
        """[summary]

        Args:
            a (int): [description]

        Returns:
            int: [description]
        """
        pass
